import json
import os
import re
import sys

from oslo_config import cfg
from subprocess import check_output

from st2actions.runners.pythonrunner import Action
from st2common.util.config_parser import ContentPackConfigParser


class GetSanitizedActionsAction(Action):

    def run(self, subtype=None):
        if not os.path.exists('/usr/bin/st2'):
            raise Exception('The stackstorm client could not be found in /usr/bin/st2.')
        username = self.config['st2_username']
        password = self.config['st2_password']

        # Get list of all packs
        pack_dir = os.path.join(cfg.CONF.system.base_path, 'packs/')
        _, self._all_packs, _ = os.walk(pack_dir).next()

        # Get st2 auth token
        self._st2_auth_token = check_output('/usr/bin/st2 auth {} -p {} -t'.format(username, password),
                                            shell=True).strip()

        pack_list = self._get_packs_for_subtype(subtype)
        self.logger.debug('pack list: {}'.format(pack_list))

        pack_data = []
        for pack in pack_list:
            pack_actions = self._get_actions_for_pack(username, password, pack)
            if pack_actions:
                pack_data += pack_actions
        return json.dumps(pack_data)

    def _get_packs_for_subtype(self, subtype):
        # @TODO: do we need to support packs with multiple subtypes?
        pack_list = []
        # Always include generic subtype
        subtypes = ['generic']

        if subtype:
            # If subtype is a custom XDS schema, return generic + pack actions
            match = re.match(r'^XDS-(.*) Schema$', subtype)
            if match:
                self.logger.debug('Custom XDS type specified: {}'.format(match.group(1)))
                pack_list.append(match.group(1).lower())
            # If subtype is not custom, return generic + subtype matching packs
            else:
                self.logger.debug('Retrieving actions for subtype {}'.format(subtype))
                subtypes.append(subtype.lower())

        for pack in self._all_packs:
            # Check pack for a config.yaml
            parser = ContentPackConfigParser(pack_name=pack)
            pack_config = parser.get_config()
            # If the current pack's config file is missing or empty, skip it
            if not pack_config or not pack_config.config:
                continue
            # Look for the subtype field in config.yaml
            pack_subtypes = [x.lower() for x in pack_config.config.get('subtypes', [])]
            # If subtype matches, get actions for this pack
            #if pack_subtype and pack_subtype.lower() in subtypes:
            if pack_subtypes:
                if set(pack_subtypes).intersection(subtypes):
                    pack_list.append(pack)

        return pack_list

    def _get_actions_for_pack(self, username, password, pack):
        if pack not in self._all_packs:
            return None

        # Set auth token as an env var
        env = dict(os.environ, ST2_AUTH_TOKEN=self._st2_auth_token)
        
        # Get json list of actions filtered by pack
        cmd = '/usr/bin/st2 action list -a name ref parameters -p {} -j'.format(pack)
        action_json = check_output(cmd, env=env, shell=True)
        try:
            data = json.loads(action_json)
        except ValueError:
            data = {}

        # Sanitize list of actions
        new_data = []
        for data_dict in data:
            new_dict = {}
            for k, v in data_dict.iteritems():
                # Remove "default" field for parameters - there is often template logic in
                # here that breaks workflows.
                if k == 'parameters':
                    for param_name, param_dict in v.items():
                        if 'default' in param_dict:
                            del param_dict['default']
                # Rename "ref" to "id"
                if k == 'ref':
                    new_dict['id'] = v
                else:
                    new_dict[k] = v
            new_data.append(new_dict)
        return new_data
