import cx_Oracle
from st2reactor.sensor.base import PollingSensor
from st2client.client import Client
from st2client.models import KeyValuePair
class DeploymentSensor(PollingSensor):
    def setup(self):
        self._st2_client = Client(base_url='http://localhost')
        self._last_deploy_timestamp = None
        last_deploy_key = self._st2_client.keys.get_by_name('udeploy_last_deployment')
        if last_deploy_key:
            # Initialize the last timestamp if it exists in the datastore at startup
            self._last_deploy_timestamp = last_deploy_key.value
        oracle_host = self.config.get('oracle_host')
        # @TODO: make sure this works with string vs int
        oracle_port = self.config.get('oracle_port') or 1521
        oracle_sid = self.config.get('oracle_sid')
        dsn_tns = cx_Oracle.makedsn(oracle_host, oracle_port, oracle_sid)
        username = self.config.get('oracle_username')
        password = self.config.get('oracle_password')
        db = cx_Oracle.connect(username, password, dsn_tns)
        self._cursor = db.cursor()
    def poll(self):
        app_id_str = self.get_app_id_str()
        last_timestamp = self.last_deploy_timestamp
        # If there is a set timestamp, get results since then.
        if last_timestamp:
            self.get_deployments_since(app_id_str, last_timestamp)
        # If not, get the most recent result and use that to set a timestamp.
        else:
            self.get_last_deployment(app_id_str)
    @property
    def last_deploy_timestamp(self):
        return self._last_deploy_timestamp
    @last_deploy_timestamp.setter
    def last_deploy_timestamp(self, timestamp):
        """
        This is the setter for the last deployment timestamp. It manages both the in-memory timestamp
        (self._last_deploy_timestamp) and the datastore timestamp (system.udeploy_last_deployment).
        """
        timestamp = str(timestamp)
        # If timestamp is not set, use this value
        if not self._last_deploy_timestamp:
            self._last_deploy_timestamp = timestamp
        # Otherwise, use this value if it's more recent than the in-memory timestamp
        else:
            if int(timestamp) > int(self._last_deploy_timestamp):
                self._last_deploy_timestamp = timestamp
        # Check against the datastore timestamp
        last_deploy_key = self._st2_client.keys.get_by_name('udeploy_last_deployment')
        if not last_deploy_key:
            last_deploy_key = KeyValuePair(name='udeploy_last_deployment',
                                           value=self._last_deploy_timestamp)
        else:
            # Sync the datastore timestamp and the in-memory timestamp
            if int(last_deploy_key.value) < int(self._last_deploy_timestamp):
                last_deploy_key.value = self._last_deploy_timestamp
            else:
                if int(last_deploy_key.value) > int(self._last_deploy_timestamp):
                    self._last_deploy_timestamp = last_deploy_key.value
                # Return here to avoid an unnecessary update (used for testing)
                return
        self._st2_client.keys.update(last_deploy_key)
    def get_app_id_str(self):
        app_ids = self.config.get('app_ids', [])
        # Escape each app id with quotes and join in a comma separated string.
        return ", ".join(["\'{}\'".format(x) for x in app_ids])
    def get_deployments_since(self, app_id_str, timestamp):
        query = ('SELECT RT_APP_PROCESS_REQUEST.RESULT AS STATUS, RT_APP_PROCESS_REQUEST.SUBMITTED_TIME AS SUBMITTED_TIME, RT_APP_PROCESS_REQUEST.ID AS REQUEST, DS_ENVIRONMENT.NAME AS ENVIRONMENT, DS_VERSION.NAME AS VERSION, DS_COMPONENT.NAME AS COMPONENT, SEC_USER.NAME AS USERID, WF_WORKFLOW_METADATA.DURATION_TIME AS DURATION, DS_APPLICATION.NAME AS APP ' +
                 'from RT_APP_PROCESS_REQUEST ' +
		 ' INNER JOIN RT_APP_PROC_REQ_TO_VERSION ON RT_APP_PROCESS_REQUEST.ID = RT_APP_PROC_REQ_TO_VERSION.APP_PROCESS_REQUEST_ID INNER JOIN DS_VERSION ON RT_APP_PROC_REQ_TO_VERSION.VERSION_ID = DS_VERSION.ID INNER JOIN DS_COMPONENT ON DS_VERSION.COMPONENT_ID = DS_COMPONENT.ID INNER JOIN SEC_USER ON RT_APP_PROCESS_REQUEST.USER_ID = SEC_USER.ID INNER JOIN DS_ENVIRONMENT ON RT_APP_PROCESS_REQUEST.ENVIRONMENT_ID=DS_ENVIRONMENT.ID INNER JOIN WF_WORKFLOW_METADATA  ON RT_APP_PROCESS_REQUEST.TRACE_ID = WF_WORKFLOW_METADATA.WORKFLOW_TRACE_ID INNER JOIN DS_APPLICATION ON RT_APP_PROCESS_REQUEST.APPLICATION_ID = DS_APPLICATION.ID ' +
                 'where RT_APP_PROCESS_REQUEST.APPLICATION_ID in ({}) and RT_APP_PROCESS_REQUEST.SUBMITTED_TIME > {} and ROWNUM <= 100 ' +
                 'order by RT_APP_PROCESS_REQUEST.SUBMITTED_TIME desc').format(app_id_str, timestamp)
        self._cursor.execute(query)
        results = self._cursor.fetchall()
        if results:
            for result in results:
                status, latest_time, _, _, _, _, _, duration, _ = result
		if status is None:
			listResult = list(result)
			listResult[0] = 'RUNNING'
			listResult[7] = '0'
			newResult = tuple(listResult)
			self.last_deploy_timestamp = latest_time
                	self.trigger(newResult)
		else:
                	self.last_deploy_timestamp = latest_time
                	self.trigger(result)
    def get_last_deployment(self, app_id_str):
        query = ('select * from (' +
                 'select ID, APPLICATION_ID, SUBMITTED_TIME ' +
                 'from RT_APP_PROCESS_REQUEST ' +
                 'where APPLICATION_ID in ({}) and ROWNUM <= 100 ' +
                 'order by SUBMITTED_TIME desc' +
                 ') WHERE ROWNUM <= 1').format(app_id_str)
        self._cursor.execute(query)
        results = self._cursor.fetchall()
        if results:
            _, _, latest_time = results[0]
            self.last_deploy_timestamp = latest_time
            self.trigger(results[0])
    def trigger(self, result):
        self.sensor_service.dispatch('udeploy.udeploy_new_deployments',
                                     self._to_payload(result))
    def _to_payload(self, result):
        STATUS, SUBMITTED_TIME, REQUEST, ENVIRONMENT, VERSION, COMPONENT, USERID, DURATION, APP = result
        return {'status': STATUS, 'submitted_time': SUBMITTED_TIME, 'request': REQUEST, 'environment': ENVIRONMENT, 'version': VERSION, 'component': COMPONENT, 'userid': USERID, 'duration': DURATION, 'app': APP}
    def cleanup(self):
        # This is called when the st2 system goes down. You can perform cleanup operations like
        # closing the connections to external system here.
        pass
    def add_trigger(self, trigger):
        # This method is called when trigger is created
        pass
    def update_trigger(self, trigger):
        # This method is called when trigger is updated
        pass
    def remove_trigger(self, trigger):
        # This method is called when trigger is deleted
	
        pass

